{% extends "!layout.html" %}

{% block menu %}
    {% if include_index_link %}
    <a href="https://azure.github.io/azure-sdk-for-python">Return to Index</a>
    <p class="caption" id="versionSelectorHeader" style="display: none;">
        Package version
    <br />
    <select value="myValue" id="versionSelector" style="display: none;"></select>
    <br />
    </p>
    <br />
    {% endif %}
    {{ super() }}
{% endblock %}

{% block footer %}
{{ super() }}
<!-- Global site tag (gtag.js) - Google Analytics -->
<script async src="https://www.googletagmanager.com/gtag/js?id=UA-62780441-36"></script>
<script>
  window.dataLayer = window.dataLayer || [];
  function gtag(){dataLayer.push(arguments);}
  gtag('js', new Date());

  gtag('config', 'UA-62780441-36');
</script>
{% endblock %}