# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.dellstorage import DellStorageMgmtClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDellStorageMgmtFileSystemsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(DellStorageMgmtClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_file_systems_get(self, resource_group):
        response = self.client.file_systems.get(
            resource_group_name=resource_group.name,
            filesystem_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_file_systems_begin_create_or_update(self, resource_group):
        response = self.client.file_systems.begin_create_or_update(
            resource_group_name=resource_group.name,
            filesystem_name="str",
            resource={
                "location": "str",
                "id": "str",
                "identity": {
                    "type": "str",
                    "principalId": "str",
                    "tenantId": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "name": "str",
                "properties": {
                    "delegatedSubnetCidr": "str",
                    "delegatedSubnetId": "str",
                    "dellReferenceNumber": "str",
                    "encryption": {
                        "encryptionType": "str",
                        "encryptionIdentityProperties": {"identityResourceId": "str", "identityType": "str"},
                        "keyUrl": "str",
                    },
                    "marketplace": {
                        "offerId": "str",
                        "planId": "str",
                        "planName": "str",
                        "endDate": "str",
                        "marketplaceSubscriptionId": "str",
                        "marketplaceSubscriptionStatus": "str",
                        "privateOfferId": "str",
                        "publisherId": "str",
                        "termUnit": "str",
                    },
                    "user": {"email": "str"},
                    "capacity": {"current": "str", "incremental": "str", "max": "str", "min": "str"},
                    "fileSystemId": "str",
                    "oneFsUrl": "str",
                    "provisioningState": "str",
                    "smartConnectFqdn": "str",
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_file_systems_update(self, resource_group):
        response = self.client.file_systems.update(
            resource_group_name=resource_group.name,
            filesystem_name="str",
            properties={
                "identity": {
                    "type": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "properties": {
                    "capacity": {"current": "str", "incremental": "str", "max": "str", "min": "str"},
                    "delegatedSubnetId": "str",
                    "encryption": {
                        "encryptionIdentityProperties": {"identityResourceId": "str", "identityType": "str"},
                        "encryptionType": "str",
                        "keyUrl": "str",
                    },
                },
                "tags": {"str": "str"},
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_file_systems_begin_delete(self, resource_group):
        response = self.client.file_systems.begin_delete(
            resource_group_name=resource_group.name,
            filesystem_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_file_systems_list_by_resource_group(self, resource_group):
        response = self.client.file_systems.list_by_resource_group(
            resource_group_name=resource_group.name,
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_file_systems_list_by_subscription(self, resource_group):
        response = self.client.file_systems.list_by_subscription()
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...
