# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._backup_vaults_operations import BackupVaultsOperations
from ._operation_result_operations import OperationResultOperations
from ._operation_status_operations import OperationStatusOperations
from ._operation_status_backup_vault_context_operations import OperationStatusBackupVaultContextOperations
from ._operation_status_resource_group_context_operations import OperationStatusResourceGroupContextOperations
from ._backup_vault_operation_results_operations import BackupVaultOperationResultsOperations
from ._data_protection_operations import DataProtectionOperations
from ._data_protection_operations_operations import DataProtectionOperationsOperations
from ._backup_policies_operations import BackupPoliciesOperations
from ._backup_instances_operations import BackupInstancesOperations
from ._recovery_points_operations import RecoveryPointsOperations
from ._fetch_secondary_recovery_points_operations import FetchSecondaryRecoveryPointsOperations
from ._fetch_cross_region_restore_job_operations import FetchCrossRegionRestoreJobOperations
from ._fetch_cross_region_restore_jobs_operations import FetchCrossRegionRestoreJobsOperations
from ._backup_instances_extension_routing_operations import BackupInstancesExtensionRoutingOperations
from ._jobs_operations import JobsOperations
from ._restorable_time_ranges_operations import RestorableTimeRangesOperations
from ._export_jobs_operations import ExportJobsOperations
from ._export_jobs_operation_result_operations import ExportJobsOperationResultOperations
from ._deleted_backup_instances_operations import DeletedBackupInstancesOperations
from ._resource_guards_operations import ResourceGuardsOperations
from ._dpp_resource_guard_proxy_operations import DppResourceGuardProxyOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "BackupVaultsOperations",
    "OperationResultOperations",
    "OperationStatusOperations",
    "OperationStatusBackupVaultContextOperations",
    "OperationStatusResourceGroupContextOperations",
    "BackupVaultOperationResultsOperations",
    "DataProtectionOperations",
    "DataProtectionOperationsOperations",
    "BackupPoliciesOperations",
    "BackupInstancesOperations",
    "RecoveryPointsOperations",
    "FetchSecondaryRecoveryPointsOperations",
    "FetchCrossRegionRestoreJobOperations",
    "FetchCrossRegionRestoreJobsOperations",
    "BackupInstancesExtensionRoutingOperations",
    "JobsOperations",
    "RestorableTimeRangesOperations",
    "ExportJobsOperations",
    "ExportJobsOperationResultOperations",
    "DeletedBackupInstancesOperations",
    "ResourceGuardsOperations",
    "DppResourceGuardProxyOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
