from setuptools import setup

setup(
    name="breaking-changes-checker",
    version="0.0.1",
    url='http://github.com/Azure/azure-sdk-for-python',
    license='MIT License',
    description="A tool which enforces azure sdk guidelines.",
    author='Microsoft Corporation',
    author_email='azpysdkhelp@microsoft.com',
    py_modules=['breaking_changes_checker'],
)