# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

VERSION = "1.2.1"

SDK_MONIKER = "digitaltwins-core/{}".format(VERSION)  # type: str
