# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._operations import Operations
from ._views_operations import ViewsOperations
from ._alerts_operations import AlertsOperations
from ._forecast_operations import ForecastOperations
from ._dimensions_operations import DimensionsOperations
from ._query_operations import QueryOperations
from ._generate_reservation_details_report_operations import GenerateReservationDetailsReportOperations
from ._exports_operations import ExportsOperations
from ._generate_cost_details_report_operations import GenerateCostDetailsReportOperations
from ._generate_detailed_cost_report_operations import GenerateDetailedCostReportOperations
from ._generate_detailed_cost_report_operation_results_operations import (
    GenerateDetailedCostReportOperationResultsOperations,
)
from ._generate_detailed_cost_report_operation_status_operations import (
    GenerateDetailedCostReportOperationStatusOperations,
)
from ._price_sheet_operations import PriceSheetOperations
from ._scheduled_actions_operations import ScheduledActionsOperations
from ._benefit_recommendations_operations import BenefitRecommendationsOperations
from ._benefit_utilization_summaries_operations import BenefitUtilizationSummariesOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "ViewsOperations",
    "AlertsOperations",
    "ForecastOperations",
    "DimensionsOperations",
    "QueryOperations",
    "GenerateReservationDetailsReportOperations",
    "ExportsOperations",
    "GenerateCostDetailsReportOperations",
    "GenerateDetailedCostReportOperations",
    "GenerateDetailedCostReportOperationResultsOperations",
    "GenerateDetailedCostReportOperationStatusOperations",
    "PriceSheetOperations",
    "ScheduledActionsOperations",
    "BenefitRecommendationsOperations",
    "BenefitUtilizationSummariesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
