# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._arc_settings_operations import ArcSettingsOperations
from ._clusters_operations import ClustersOperations
from ._deployment_settings_operations import DeploymentSettingsOperations
from ._edge_devices_operations import EdgeDevicesOperations
from ._extensions_operations import ExtensionsOperations
from ._offers_operations import OffersOperations
from ._operations import Operations
from ._publishers_operations import PublishersOperations
from ._security_settings_operations import SecuritySettingsOperations
from ._skus_operations import SkusOperations
from ._update_runs_operations import UpdateRunsOperations
from ._update_summaries_operations import UpdateSummariesOperations
from ._updates_operations import UpdatesOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ArcSettingsOperations",
    "ClustersOperations",
    "DeploymentSettingsOperations",
    "EdgeDevicesOperations",
    "ExtensionsOperations",
    "OffersOperations",
    "Operations",
    "PublishersOperations",
    "SecuritySettingsOperations",
    "SkusOperations",
    "UpdateRunsOperations",
    "UpdateSummariesOperations",
    "UpdatesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
