# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import Operations  # type: ignore
from ._operations import AvailabilitySetsOperations  # type: ignore
from ._operations import ProximityPlacementGroupsOperations  # type: ignore
from ._operations import DedicatedHostGroupsOperations  # type: ignore
from ._operations import DedicatedHostsOperations  # type: ignore
from ._operations import SshPublicKeysOperations  # type: ignore
from ._operations import VirtualMachineExtensionImagesOperations  # type: ignore
from ._operations import VirtualMachineExtensionsOperations  # type: ignore
from ._operations import VirtualMachineImagesOperations  # type: ignore
from ._operations import VirtualMachineImagesEdgeZoneOperations  # type: ignore
from ._operations import UsageOperations  # type: ignore
from ._operations import VirtualMachinesOperations  # type: ignore
from ._operations import VirtualMachineScaleSetsOperations  # type: ignore
from ._operations import VirtualMachineSizesOperations  # type: ignore
from ._operations import ImagesOperations  # type: ignore
from ._operations import RestorePointCollectionsOperations  # type: ignore
from ._operations import RestorePointsOperations  # type: ignore
from ._operations import CapacityReservationGroupsOperations  # type: ignore
from ._operations import CapacityReservationsOperations  # type: ignore
from ._operations import VirtualMachineScaleSetExtensionsOperations  # type: ignore
from ._operations import VirtualMachineScaleSetRollingUpgradesOperations  # type: ignore
from ._operations import VirtualMachineScaleSetVMExtensionsOperations  # type: ignore
from ._operations import VirtualMachineScaleSetVMsOperations  # type: ignore
from ._operations import LogAnalyticsOperations  # type: ignore
from ._operations import VirtualMachineRunCommandsOperations  # type: ignore
from ._operations import VirtualMachineScaleSetVMRunCommandsOperations  # type: ignore
from ._operations import ResourceSkusOperations  # type: ignore
from ._operations import GalleriesOperations  # type: ignore
from ._operations import GalleryImagesOperations  # type: ignore
from ._operations import GalleryImageVersionsOperations  # type: ignore
from ._operations import GalleryApplicationsOperations  # type: ignore
from ._operations import GalleryApplicationVersionsOperations  # type: ignore
from ._operations import GallerySharingProfileOperations  # type: ignore
from ._operations import SharedGalleriesOperations  # type: ignore
from ._operations import SharedGalleryImagesOperations  # type: ignore
from ._operations import SharedGalleryImageVersionsOperations  # type: ignore
from ._operations import CommunityGalleriesOperations  # type: ignore
from ._operations import CommunityGalleryImagesOperations  # type: ignore
from ._operations import CommunityGalleryImageVersionsOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "AvailabilitySetsOperations",
    "ProximityPlacementGroupsOperations",
    "DedicatedHostGroupsOperations",
    "DedicatedHostsOperations",
    "SshPublicKeysOperations",
    "VirtualMachineExtensionImagesOperations",
    "VirtualMachineExtensionsOperations",
    "VirtualMachineImagesOperations",
    "VirtualMachineImagesEdgeZoneOperations",
    "UsageOperations",
    "VirtualMachinesOperations",
    "VirtualMachineScaleSetsOperations",
    "VirtualMachineSizesOperations",
    "ImagesOperations",
    "RestorePointCollectionsOperations",
    "RestorePointsOperations",
    "CapacityReservationGroupsOperations",
    "CapacityReservationsOperations",
    "VirtualMachineScaleSetExtensionsOperations",
    "VirtualMachineScaleSetRollingUpgradesOperations",
    "VirtualMachineScaleSetVMExtensionsOperations",
    "VirtualMachineScaleSetVMsOperations",
    "LogAnalyticsOperations",
    "VirtualMachineRunCommandsOperations",
    "VirtualMachineScaleSetVMRunCommandsOperations",
    "ResourceSkusOperations",
    "GalleriesOperations",
    "GalleryImagesOperations",
    "GalleryImageVersionsOperations",
    "GalleryApplicationsOperations",
    "GalleryApplicationVersionsOperations",
    "GallerySharingProfileOperations",
    "SharedGalleriesOperations",
    "SharedGalleryImagesOperations",
    "SharedGalleryImageVersionsOperations",
    "CommunityGalleriesOperations",
    "CommunityGalleryImagesOperations",
    "CommunityGalleryImageVersionsOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
