# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils.aio import recorded_by_proxy_async
from testpreparer import AIProjectPreparer
from testpreparer_async import AIProjectClientTestBaseAsync


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAIProjectEvaluationResultsOperationsAsync(AIProjectClientTestBaseAsync):
    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_list_versions(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = client.evaluation_results.list_versions(
            name="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_list_latest(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = client.evaluation_results.list_latest()
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_get_version(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = await client.evaluation_results.get_version(
            name="str",
            version="str",
        )

        # please add some check logic here by yourself
        # ...

    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_delete_version(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = await client.evaluation_results.delete_version(
            name="str",
            version="str",
        )

        # please add some check logic here by yourself
        # ...

    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_create(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = await client.evaluation_results.create(
            name="str",
            body={
                "name": "str",
                "version": "str",
                "BlobUri": "str",
                "DatasetFamily": "str",
                "DatasetName": "str",
                "Metrics": {"str": 0.0},
                "ModelAssetId": "str",
                "ModelName": "str",
                "ModelVersion": "str",
                "ResultType": "str",
                "description": "str",
                "id": "str",
                "stage": "str",
                "tags": {"str": "str"},
            },
        )

        # please add some check logic here by yourself
        # ...

    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_create_version(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = await client.evaluation_results.create_version(
            name="str",
            version="str",
            body={
                "name": "str",
                "version": "str",
                "BlobUri": "str",
                "DatasetFamily": "str",
                "DatasetName": "str",
                "Metrics": {"str": 0.0},
                "ModelAssetId": "str",
                "ModelName": "str",
                "ModelVersion": "str",
                "ResultType": "str",
                "description": "str",
                "id": "str",
                "stage": "str",
                "tags": {"str": "str"},
            },
        )

        # please add some check logic here by yourself
        # ...

    @AIProjectPreparer()
    @recorded_by_proxy_async
    async def test_evaluation_results_start_pending_upload(self, aiproject_endpoint):
        client = self.create_async_client(endpoint=aiproject_endpoint)
        response = await client.evaluation_results.start_pending_upload(
            name="str",
            version="str",
            body={"pendingUploadType": "str", "connectionName": "str", "pendingUploadId": "str"},
        )

        # please add some check logic here by yourself
        # ...
