# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._service_topologies_operations import ServiceTopologiesOperations
from ._services_operations import ServicesOperations
from ._service_units_operations import ServiceUnitsOperations
from ._steps_operations import StepsOperations
from ._rollouts_operations import RolloutsOperations
from ._artifact_sources_operations import ArtifactSourcesOperations
from ._operations import Operations

from ._patch import __all__ as _patch_all
from ._patch import *  # type: ignore # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ServiceTopologiesOperations",
    "ServicesOperations",
    "ServiceUnitsOperations",
    "StepsOperations",
    "RolloutsOperations",
    "ArtifactSourcesOperations",
    "Operations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
