# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.compute import ComputeManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestComputeManagementImagesOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ComputeManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_images_begin_create_or_update(self, resource_group):
        response = self.client.images.begin_create_or_update(
            resource_group_name=resource_group.name,
            image_name="str",
            parameters={
                "location": "str",
                "extendedLocation": {"name": "str", "type": "str"},
                "hyperVGeneration": "str",
                "id": "str",
                "name": "str",
                "provisioningState": "str",
                "sourceVirtualMachine": {"id": "str"},
                "storageProfile": {
                    "dataDisks": [
                        {
                            "lun": 0,
                            "blobUri": "str",
                            "caching": "str",
                            "diskEncryptionSet": {"id": "str"},
                            "diskSizeGB": 0,
                            "managedDisk": {"id": "str"},
                            "snapshot": {"id": "str"},
                            "storageAccountType": "str",
                        }
                    ],
                    "osDisk": {
                        "osState": "str",
                        "osType": "str",
                        "blobUri": "str",
                        "caching": "str",
                        "diskEncryptionSet": {"id": "str"},
                        "diskSizeGB": 0,
                        "managedDisk": {"id": "str"},
                        "snapshot": {"id": "str"},
                        "storageAccountType": "str",
                    },
                    "zoneResilient": bool,
                },
                "tags": {"str": "str"},
                "type": "str",
            },
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_images_begin_update(self, resource_group):
        response = self.client.images.begin_update(
            resource_group_name=resource_group.name,
            image_name="str",
            parameters={
                "hyperVGeneration": "str",
                "provisioningState": "str",
                "sourceVirtualMachine": {"id": "str"},
                "storageProfile": {
                    "dataDisks": [
                        {
                            "lun": 0,
                            "blobUri": "str",
                            "caching": "str",
                            "diskEncryptionSet": {"id": "str"},
                            "diskSizeGB": 0,
                            "managedDisk": {"id": "str"},
                            "snapshot": {"id": "str"},
                            "storageAccountType": "str",
                        }
                    ],
                    "osDisk": {
                        "osState": "str",
                        "osType": "str",
                        "blobUri": "str",
                        "caching": "str",
                        "diskEncryptionSet": {"id": "str"},
                        "diskSizeGB": 0,
                        "managedDisk": {"id": "str"},
                        "snapshot": {"id": "str"},
                        "storageAccountType": "str",
                    },
                    "zoneResilient": bool,
                },
                "tags": {"str": "str"},
            },
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_images_begin_delete(self, resource_group):
        response = self.client.images.begin_delete(
            resource_group_name=resource_group.name,
            image_name="str",
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_images_get(self, resource_group):
        response = self.client.images.get(
            resource_group_name=resource_group.name,
            image_name="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_images_list_by_resource_group(self, resource_group):
        response = self.client.images.list_by_resource_group(
            resource_group_name=resource_group.name,
            api_version="2024-11-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_images_list(self, resource_group):
        response = self.client.images.list(
            api_version="2024-11-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...
