# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import Operations  # type: ignore
from ._annotations_operations import AnnotationsOperations  # type: ignore
from ._api_keys_operations import APIKeysOperations  # type: ignore
from ._export_configurations_operations import ExportConfigurationsOperations  # type: ignore
from ._component_current_billing_features_operations import ComponentCurrentBillingFeaturesOperations  # type: ignore
from ._component_quota_status_operations import ComponentQuotaStatusOperations  # type: ignore
from ._component_feature_capabilities_operations import ComponentFeatureCapabilitiesOperations  # type: ignore
from ._component_available_features_operations import ComponentAvailableFeaturesOperations  # type: ignore
from ._proactive_detection_configurations_operations import ProactiveDetectionConfigurationsOperations  # type: ignore
from ._components_operations import ComponentsOperations  # type: ignore
from ._work_item_configurations_operations import WorkItemConfigurationsOperations  # type: ignore
from ._favorites_operations import FavoritesOperations  # type: ignore
from ._web_test_locations_operations import WebTestLocationsOperations  # type: ignore
from ._web_tests_operations import WebTestsOperations  # type: ignore
from ._analytics_items_operations import AnalyticsItemsOperations  # type: ignore
from ._workbooks_operations import WorkbooksOperations  # type: ignore
from ._my_workbooks_operations import MyWorkbooksOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "AnnotationsOperations",
    "APIKeysOperations",
    "ExportConfigurationsOperations",
    "ComponentCurrentBillingFeaturesOperations",
    "ComponentQuotaStatusOperations",
    "ComponentFeatureCapabilitiesOperations",
    "ComponentAvailableFeaturesOperations",
    "ProactiveDetectionConfigurationsOperations",
    "ComponentsOperations",
    "WorkItemConfigurationsOperations",
    "FavoritesOperations",
    "WebTestLocationsOperations",
    "WebTestsOperations",
    "AnalyticsItemsOperations",
    "WorkbooksOperations",
    "MyWorkbooksOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
