# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models import AddOrUpdateTextBlocklistItemsOptions
from ._models import AddOrUpdateTextBlocklistItemsResult
from ._models import AnalyzeImageOptions
from ._models import AnalyzeImageResult
from ._models import AnalyzeTextOptions
from ._models import AnalyzeTextResult
from ._models import ImageCategoriesAnalysis
from ._models import ImageData
from ._models import RemoveTextBlocklistItemsOptions
from ._models import TextBlocklist
from ._models import TextBlocklistItem
from ._models import TextBlocklistMatch
from ._models import TextCategoriesAnalysis

from ._enums import AnalyzeImageOutputType
from ._enums import AnalyzeTextOutputType
from ._enums import ImageCategory
from ._enums import TextCategory
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AddOrUpdateTextBlocklistItemsOptions",
    "AddOrUpdateTextBlocklistItemsResult",
    "AnalyzeImageOptions",
    "AnalyzeImageResult",
    "AnalyzeTextOptions",
    "AnalyzeTextResult",
    "ImageCategoriesAnalysis",
    "ImageData",
    "RemoveTextBlocklistItemsOptions",
    "TextBlocklist",
    "TextBlocklistItem",
    "TextBlocklistMatch",
    "TextCategoriesAnalysis",
    "AnalyzeImageOutputType",
    "AnalyzeTextOutputType",
    "ImageCategory",
    "TextCategory",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
