# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class ActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum. Indicates the action type. "Internal" refers to actions that are for internal only APIs."""

    INTERNAL = "Internal"


class CreatedByType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of identity that created the resource."""

    USER = "User"
    APPLICATION = "Application"
    MANAGED_IDENTITY = "ManagedIdentity"
    KEY = "Key"


class Origin(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The intended executor of the operation; as in Resource Based Access Control (RBAC) and audit
    logs UX. Default value is "user,system".
    """

    USER = "user"
    SYSTEM = "system"
    USER_SYSTEM = "user,system"


class ResourceState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Resources provisioning states."""

    NOT_SPECIFIED = "NotSpecified"
    ACCEPTED = "Accepted"
    CREATING = "Creating"
    SUCCEEDED = "Succeeded"
    FAILED = "Failed"
    CANCELED = "Canceled"
    PROVISIONING_RESOURCES = "ProvisioningResources"
    INSTALLING_APPLICATION = "InstallingApplication"
    CONFIGURING_APPLICATION = "ConfiguringApplication"
    MIGRATING_APPLICATION_DATA = "MigratingApplicationData"
    RUNNING_VALIDATIONS = "RunningValidations"
    CREATING_ARTIFACTS = "CreatingArtifacts"
    DELETING_ARTIFACTS = "DeletingArtifacts"
