# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Awaitable, TYPE_CHECKING
from typing_extensions import Self

from azure.core.pipeline import policies
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.mgmt.core import AsyncARMPipelineClient
from azure.mgmt.core.policies import AsyncARMAutoResourceProviderRegistrationPolicy

from .._serialization import Deserializer, Serializer
from ._configuration import ConnectedCacheMgmtClientConfiguration
from .operations import (
    CacheNodesOperationsOperations,
    EnterpriseCustomerOperationsOperations,
    EnterpriseMccCacheNodesOperationsOperations,
    EnterpriseMccCustomersOperations,
    IspCacheNodesOperationsOperations,
    IspCustomersOperations,
    Operations,
)

if TYPE_CHECKING:
    from azure.core.credentials_async import AsyncTokenCredential


class ConnectedCacheMgmtClient:  # pylint: disable=too-many-instance-attributes
    """Microsoft Connected Cache Rest Api version 2023-05-01-preview.

    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.connectedcache.aio.operations.Operations
    :ivar enterprise_customer_operations: EnterpriseCustomerOperationsOperations operations
    :vartype enterprise_customer_operations:
     azure.mgmt.connectedcache.aio.operations.EnterpriseCustomerOperationsOperations
    :ivar cache_nodes_operations: CacheNodesOperationsOperations operations
    :vartype cache_nodes_operations:
     azure.mgmt.connectedcache.aio.operations.CacheNodesOperationsOperations
    :ivar isp_customers: IspCustomersOperations operations
    :vartype isp_customers: azure.mgmt.connectedcache.aio.operations.IspCustomersOperations
    :ivar isp_cache_nodes_operations: IspCacheNodesOperationsOperations operations
    :vartype isp_cache_nodes_operations:
     azure.mgmt.connectedcache.aio.operations.IspCacheNodesOperationsOperations
    :ivar enterprise_mcc_customers: EnterpriseMccCustomersOperations operations
    :vartype enterprise_mcc_customers:
     azure.mgmt.connectedcache.aio.operations.EnterpriseMccCustomersOperations
    :ivar enterprise_mcc_cache_nodes_operations: EnterpriseMccCacheNodesOperationsOperations
     operations
    :vartype enterprise_mcc_cache_nodes_operations:
     azure.mgmt.connectedcache.aio.operations.EnterpriseMccCacheNodesOperationsOperations
    :param credential: Credential used to authenticate requests to the service. Required.
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential
    :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
    :type subscription_id: str
    :param base_url: Service host. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: The API version to use for this operation. Default value is
     "2023-05-01-preview". Note that overriding this default value may result in unsupported
     behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "AsyncTokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        _endpoint = "{endpoint}"
        self._config = ConnectedCacheMgmtClientConfiguration(
            credential=credential, subscription_id=subscription_id, base_url=base_url, **kwargs
        )
        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                AsyncARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: AsyncARMPipelineClient = AsyncARMPipelineClient(base_url=_endpoint, policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.enterprise_customer_operations = EnterpriseCustomerOperationsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.cache_nodes_operations = CacheNodesOperationsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.isp_customers = IspCustomersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.isp_cache_nodes_operations = IspCacheNodesOperationsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.enterprise_mcc_customers = EnterpriseMccCustomersOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.enterprise_mcc_cache_nodes_operations = EnterpriseMccCacheNodesOperationsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def send_request(
        self, request: HttpRequest, *, stream: bool = False, **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client.send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.base_url", self._config.base_url, "str", skip_quote=True),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> Self:
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)
