# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class ProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Provisioning state of the Azure Dev Spaces Controller."""

    SUCCEEDED = "Succeeded"
    FAILED = "Failed"
    CANCELED = "Canceled"
    UPDATING = "Updating"
    CREATING = "Creating"
    DELETING = "Deleting"
    DELETED = "Deleted"


class SkuName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The name of the SKU for Azure Dev Spaces Controller."""

    S1 = "S1"


class SkuTier(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The tier of the SKU for Azure Dev Spaces Controller."""

    STANDARD = "Standard"
