# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import Category
from ._models_py3 import CertSyncRecord
from ._models_py3 import CheckNameAvailabilityRequest
from ._models_py3 import CheckNameAvailabilityResponse
from ._models_py3 import ComplianceReportItem
from ._models_py3 import ComplianceResult
from ._models_py3 import Control
from ._models_py3 import ControlFamily
from ._models_py3 import ControlSyncRecord
from ._models_py3 import DownloadResponse
from ._models_py3 import DownloadResponseComplianceDetailedPdfReport
from ._models_py3 import DownloadResponseCompliancePdfReport
from ._models_py3 import ErrorAdditionalInfo
from ._models_py3 import ErrorDetail
from ._models_py3 import ErrorResponse
from ._models_py3 import EvidenceFileDownloadRequest
from ._models_py3 import EvidenceFileDownloadResponse
from ._models_py3 import EvidenceFileDownloadResponseEvidenceFile
from ._models_py3 import EvidenceProperties
from ._models_py3 import EvidenceResource
from ._models_py3 import EvidenceResourceListResult
from ._models_py3 import GetCollectionCountRequest
from ._models_py3 import GetCollectionCountResponse
from ._models_py3 import GetOverviewStatusRequest
from ._models_py3 import GetOverviewStatusResponse
from ._models_py3 import ListInUseStorageAccountsRequest
from ._models_py3 import ListInUseStorageAccountsResponse
from ._models_py3 import OnboardRequest
from ._models_py3 import OnboardResponse
from ._models_py3 import Operation
from ._models_py3 import OperationDisplay
from ._models_py3 import OperationListResult
from ._models_py3 import OverviewStatus
from ._models_py3 import ProxyResource
from ._models_py3 import QuickAssessment
from ._models_py3 import Recommendation
from ._models_py3 import RecommendationSolution
from ._models_py3 import ReportComplianceStatus
from ._models_py3 import ReportFixResult
from ._models_py3 import ReportPatchProperties
from ._models_py3 import ReportProperties
from ._models_py3 import ReportResource
from ._models_py3 import ReportResourceListResult
from ._models_py3 import ReportResourcePatch
from ._models_py3 import ReportVerificationResult
from ._models_py3 import Resource
from ._models_py3 import ResourceItem
from ._models_py3 import ResourceMetadata
from ._models_py3 import Responsibility
from ._models_py3 import ResponsibilityResource
from ._models_py3 import ScopingAnswer
from ._models_py3 import ScopingConfigurationProperties
from ._models_py3 import ScopingConfigurationResource
from ._models_py3 import ScopingConfigurationResourceListResult
from ._models_py3 import ScopingQuestion
from ._models_py3 import ScopingQuestions
from ._models_py3 import SnapshotDownloadRequest
from ._models_py3 import SnapshotProperties
from ._models_py3 import SnapshotResource
from ._models_py3 import SnapshotResourceListResult
from ._models_py3 import StatusItem
from ._models_py3 import StorageInfo
from ._models_py3 import SyncCertRecordRequest
from ._models_py3 import SyncCertRecordResponse
from ._models_py3 import SystemData
from ._models_py3 import TriggerEvaluationProperty
from ._models_py3 import TriggerEvaluationRequest
from ._models_py3 import TriggerEvaluationResponse
from ._models_py3 import WebhookProperties
from ._models_py3 import WebhookResource
from ._models_py3 import WebhookResourceListResult
from ._models_py3 import WebhookResourcePatch

from ._app_compliance_automation_mgmt_client_enums import ActionType
from ._app_compliance_automation_mgmt_client_enums import CategoryStatus
from ._app_compliance_automation_mgmt_client_enums import CheckNameAvailabilityReason
from ._app_compliance_automation_mgmt_client_enums import ContentType
from ._app_compliance_automation_mgmt_client_enums import ControlFamilyStatus
from ._app_compliance_automation_mgmt_client_enums import ControlStatus
from ._app_compliance_automation_mgmt_client_enums import CreatedByType
from ._app_compliance_automation_mgmt_client_enums import DeliveryStatus
from ._app_compliance_automation_mgmt_client_enums import DownloadType
from ._app_compliance_automation_mgmt_client_enums import EnableSslVerification
from ._app_compliance_automation_mgmt_client_enums import EvidenceType
from ._app_compliance_automation_mgmt_client_enums import InputType
from ._app_compliance_automation_mgmt_client_enums import IsRecommendSolution
from ._app_compliance_automation_mgmt_client_enums import NotificationEvent
from ._app_compliance_automation_mgmt_client_enums import Origin
from ._app_compliance_automation_mgmt_client_enums import ProvisioningState
from ._app_compliance_automation_mgmt_client_enums import ReportStatus
from ._app_compliance_automation_mgmt_client_enums import ResourceOrigin
from ._app_compliance_automation_mgmt_client_enums import ResourceStatus
from ._app_compliance_automation_mgmt_client_enums import ResponsibilityEnvironment
from ._app_compliance_automation_mgmt_client_enums import ResponsibilitySeverity
from ._app_compliance_automation_mgmt_client_enums import ResponsibilityStatus
from ._app_compliance_automation_mgmt_client_enums import ResponsibilityType
from ._app_compliance_automation_mgmt_client_enums import Result
from ._app_compliance_automation_mgmt_client_enums import Rule
from ._app_compliance_automation_mgmt_client_enums import SendAllEvents
from ._app_compliance_automation_mgmt_client_enums import UpdateWebhookKey
from ._app_compliance_automation_mgmt_client_enums import WebhookKeyEnabled
from ._app_compliance_automation_mgmt_client_enums import WebhookStatus
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Category",
    "CertSyncRecord",
    "CheckNameAvailabilityRequest",
    "CheckNameAvailabilityResponse",
    "ComplianceReportItem",
    "ComplianceResult",
    "Control",
    "ControlFamily",
    "ControlSyncRecord",
    "DownloadResponse",
    "DownloadResponseComplianceDetailedPdfReport",
    "DownloadResponseCompliancePdfReport",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "EvidenceFileDownloadRequest",
    "EvidenceFileDownloadResponse",
    "EvidenceFileDownloadResponseEvidenceFile",
    "EvidenceProperties",
    "EvidenceResource",
    "EvidenceResourceListResult",
    "GetCollectionCountRequest",
    "GetCollectionCountResponse",
    "GetOverviewStatusRequest",
    "GetOverviewStatusResponse",
    "ListInUseStorageAccountsRequest",
    "ListInUseStorageAccountsResponse",
    "OnboardRequest",
    "OnboardResponse",
    "Operation",
    "OperationDisplay",
    "OperationListResult",
    "OverviewStatus",
    "ProxyResource",
    "QuickAssessment",
    "Recommendation",
    "RecommendationSolution",
    "ReportComplianceStatus",
    "ReportFixResult",
    "ReportPatchProperties",
    "ReportProperties",
    "ReportResource",
    "ReportResourceListResult",
    "ReportResourcePatch",
    "ReportVerificationResult",
    "Resource",
    "ResourceItem",
    "ResourceMetadata",
    "Responsibility",
    "ResponsibilityResource",
    "ScopingAnswer",
    "ScopingConfigurationProperties",
    "ScopingConfigurationResource",
    "ScopingConfigurationResourceListResult",
    "ScopingQuestion",
    "ScopingQuestions",
    "SnapshotDownloadRequest",
    "SnapshotProperties",
    "SnapshotResource",
    "SnapshotResourceListResult",
    "StatusItem",
    "StorageInfo",
    "SyncCertRecordRequest",
    "SyncCertRecordResponse",
    "SystemData",
    "TriggerEvaluationProperty",
    "TriggerEvaluationRequest",
    "TriggerEvaluationResponse",
    "WebhookProperties",
    "WebhookResource",
    "WebhookResourceListResult",
    "WebhookResourcePatch",
    "ActionType",
    "CategoryStatus",
    "CheckNameAvailabilityReason",
    "ContentType",
    "ControlFamilyStatus",
    "ControlStatus",
    "CreatedByType",
    "DeliveryStatus",
    "DownloadType",
    "EnableSslVerification",
    "EvidenceType",
    "InputType",
    "IsRecommendSolution",
    "NotificationEvent",
    "Origin",
    "ProvisioningState",
    "ReportStatus",
    "ResourceOrigin",
    "ResourceStatus",
    "ResponsibilityEnvironment",
    "ResponsibilitySeverity",
    "ResponsibilityStatus",
    "ResponsibilityType",
    "Result",
    "Rule",
    "SendAllEvents",
    "UpdateWebhookKey",
    "WebhookKeyEnabled",
    "WebhookStatus",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
