# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import CreateResourceSupportedProperties
from ._models_py3 import CreateResourceSupportedResponse
from ._models_py3 import CreateResourceSupportedResponseList
from ._models_py3 import DatadogAgreementProperties
from ._models_py3 import DatadogAgreementResource
from ._models_py3 import DatadogAgreementResourceListResponse
from ._models_py3 import DatadogApiKey
from ._models_py3 import DatadogApiKeyListResponse
from ._models_py3 import DatadogHost
from ._models_py3 import DatadogHostListResponse
from ._models_py3 import DatadogHostMetadata
from ._models_py3 import DatadogInstallMethod
from ._models_py3 import DatadogLogsAgent
from ._models_py3 import DatadogMonitorResource
from ._models_py3 import DatadogMonitorResourceListResponse
from ._models_py3 import DatadogMonitorResourceUpdateParameters
from ._models_py3 import DatadogOrganizationProperties
from ._models_py3 import DatadogSetPasswordLink
from ._models_py3 import DatadogSingleSignOnProperties
from ._models_py3 import DatadogSingleSignOnResource
from ._models_py3 import DatadogSingleSignOnResourceListResponse
from ._models_py3 import ErrorAdditionalInfo
from ._models_py3 import ErrorDetail
from ._models_py3 import ErrorResponse
from ._models_py3 import FilteringTag
from ._models_py3 import IdentityProperties
from ._models_py3 import LinkedResource
from ._models_py3 import LinkedResourceListResponse
from ._models_py3 import LogRules
from ._models_py3 import MetricRules
from ._models_py3 import MonitorProperties
from ._models_py3 import MonitorUpdateProperties
from ._models_py3 import MonitoredResource
from ._models_py3 import MonitoredResourceListResponse
from ._models_py3 import MonitoredSubscription
from ._models_py3 import MonitoredSubscriptionProperties
from ._models_py3 import MonitoredSubscriptionPropertiesList
from ._models_py3 import MonitoringTagRules
from ._models_py3 import MonitoringTagRulesListResponse
from ._models_py3 import MonitoringTagRulesProperties
from ._models_py3 import OperationDisplay
from ._models_py3 import OperationListResult
from ._models_py3 import OperationResult
from ._models_py3 import ResourceSku
from ._models_py3 import SubscriptionList
from ._models_py3 import SystemData
from ._models_py3 import UserInfo

from ._microsoft_datadog_client_enums import CreatedByType
from ._microsoft_datadog_client_enums import LiftrResourceCategories
from ._microsoft_datadog_client_enums import ManagedIdentityTypes
from ._microsoft_datadog_client_enums import MarketplaceSubscriptionStatus
from ._microsoft_datadog_client_enums import MonitoringStatus
from ._microsoft_datadog_client_enums import Operation
from ._microsoft_datadog_client_enums import ProvisioningState
from ._microsoft_datadog_client_enums import SingleSignOnStates
from ._microsoft_datadog_client_enums import Status
from ._microsoft_datadog_client_enums import TagAction
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "CreateResourceSupportedProperties",
    "CreateResourceSupportedResponse",
    "CreateResourceSupportedResponseList",
    "DatadogAgreementProperties",
    "DatadogAgreementResource",
    "DatadogAgreementResourceListResponse",
    "DatadogApiKey",
    "DatadogApiKeyListResponse",
    "DatadogHost",
    "DatadogHostListResponse",
    "DatadogHostMetadata",
    "DatadogInstallMethod",
    "DatadogLogsAgent",
    "DatadogMonitorResource",
    "DatadogMonitorResourceListResponse",
    "DatadogMonitorResourceUpdateParameters",
    "DatadogOrganizationProperties",
    "DatadogSetPasswordLink",
    "DatadogSingleSignOnProperties",
    "DatadogSingleSignOnResource",
    "DatadogSingleSignOnResourceListResponse",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "FilteringTag",
    "IdentityProperties",
    "LinkedResource",
    "LinkedResourceListResponse",
    "LogRules",
    "MetricRules",
    "MonitorProperties",
    "MonitorUpdateProperties",
    "MonitoredResource",
    "MonitoredResourceListResponse",
    "MonitoredSubscription",
    "MonitoredSubscriptionProperties",
    "MonitoredSubscriptionPropertiesList",
    "MonitoringTagRules",
    "MonitoringTagRulesListResponse",
    "MonitoringTagRulesProperties",
    "OperationDisplay",
    "OperationListResult",
    "OperationResult",
    "ResourceSku",
    "SubscriptionList",
    "SystemData",
    "UserInfo",
    "CreatedByType",
    "LiftrResourceCategories",
    "ManagedIdentityTypes",
    "MarketplaceSubscriptionStatus",
    "MonitoringStatus",
    "Operation",
    "ProvisioningState",
    "SingleSignOnStates",
    "Status",
    "TagAction",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
