# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models_py3 import (  # type: ignore
    AbnormalTimePeriod,
    AnalysisData,
    AnalysisDefinition,
    ApiDefinitionInfo,
    AppServiceEnvironment,
    AppServicePlan,
    AppServicePlanCollection,
    ApplicationStack,
    ApplicationStackCollection,
    AutoHealActions,
    AutoHealCustomAction,
    AutoHealRules,
    AutoHealTriggers,
    BillingMeter,
    BillingMeterCollection,
    Capability,
    Certificate,
    CertificateCollection,
    CertificatePatchResource,
    CloningInfo,
    ConnStringInfo,
    CorsSettings,
    CsmMoveResourceEnvelope,
    CsmOperationCollection,
    CsmOperationDescription,
    CsmOperationDescriptionProperties,
    CsmOperationDisplay,
    CsmUsageQuota,
    CsmUsageQuotaCollection,
    DataSource,
    DataTableResponseColumn,
    DataTableResponseObject,
    DefaultErrorResponse,
    DefaultErrorResponseError,
    DefaultErrorResponseErrorDetailsItem,
    DeletedSite,
    DeletedWebAppCollection,
    DeploymentLocations,
    DetectorAbnormalTimePeriod,
    DetectorDefinition,
    DetectorInfo,
    DetectorResponse,
    DetectorResponseCollection,
    DiagnosticAnalysis,
    DiagnosticAnalysisCollection,
    DiagnosticCategory,
    DiagnosticCategoryCollection,
    DiagnosticData,
    DiagnosticDetectorCollection,
    DiagnosticDetectorResponse,
    DiagnosticMetricSample,
    DiagnosticMetricSet,
    Dimension,
    ErrorEntity,
    Experiments,
    GeoRegion,
    GeoRegionCollection,
    GlobalCsmSkuDescription,
    HandlerMapping,
    HostNameSslState,
    HostingEnvironmentDeploymentInfo,
    HostingEnvironmentProfile,
    HybridConnection,
    HybridConnectionKey,
    Identifier,
    IdentifierCollection,
    IpSecurityRestriction,
    LocalizableString,
    ManagedServiceIdentity,
    MetricAvailability,
    MetricSpecification,
    NameIdentifier,
    NameValuePair,
    NetworkAccessControlEntry,
    Operation,
    PremierAddOnOffer,
    PremierAddOnOfferCollection,
    ProxyOnlyResource,
    PushSettings,
    RampUpRule,
    Recommendation,
    RecommendationCollection,
    RecommendationRule,
    Rendering,
    RequestsBasedTrigger,
    Resource,
    ResourceHealthMetadata,
    ResourceHealthMetadataCollection,
    ResourceMetric,
    ResourceMetricAvailability,
    ResourceMetricCollection,
    ResourceMetricDefinition,
    ResourceMetricDefinitionCollection,
    ResourceMetricName,
    ResourceMetricProperty,
    ResourceMetricValue,
    ResourceNameAvailability,
    ResourceNameAvailabilityRequest,
    ResponseMetaData,
    ServiceSpecification,
    Site,
    SiteConfig,
    SiteLimits,
    SiteMachineKey,
    SkuCapacity,
    SkuDescription,
    SkuInfos,
    SlotSwapStatus,
    SlowRequestsBasedTrigger,
    SnapshotRecoveryRequest,
    SnapshotRecoveryTarget,
    Solution,
    SourceControl,
    SourceControlCollection,
    StackMajorVersion,
    StackMinorVersion,
    StampCapacity,
    StatusCodesBasedTrigger,
    User,
    ValidateRequest,
    ValidateResponse,
    ValidateResponseError,
    VirtualApplication,
    VirtualDirectory,
    VirtualIPMapping,
    VirtualNetworkProfile,
    VnetGateway,
    VnetInfo,
    VnetParameters,
    VnetRoute,
    VnetValidationFailureDetails,
    VnetValidationTestFailure,
    WebAppCollection,
    WorkerPool,
)

from ._web_site_management_client_enums import (  # type: ignore
    AccessControlEntryAction,
    AppServicePlanRestrictions,
    AutoHealActionType,
    Channels,
    CheckNameResourceTypes,
    ComputeModeOptions,
    ConnectionStringType,
    Enum0,
    Enum1,
    HostType,
    HostingEnvironmentStatus,
    InAvailabilityReasonType,
    InternalLoadBalancingMode,
    IssueType,
    KeyVaultSecretStatus,
    ManagedPipelineMode,
    ManagedServiceIdentityType,
    NotificationLevel,
    OperationStatus,
    ProvisioningState,
    RenderingType,
    ResourceScopeType,
    RouteType,
    ScmType,
    SiteAvailabilityState,
    SiteLoadBalancing,
    SkuName,
    SolutionType,
    SslState,
    StatusOptions,
    SupportedTlsVersions,
    UsageState,
    ValidateResourceTypes,
    WorkerSizeOptions,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AbnormalTimePeriod",
    "AnalysisData",
    "AnalysisDefinition",
    "ApiDefinitionInfo",
    "AppServiceEnvironment",
    "AppServicePlan",
    "AppServicePlanCollection",
    "ApplicationStack",
    "ApplicationStackCollection",
    "AutoHealActions",
    "AutoHealCustomAction",
    "AutoHealRules",
    "AutoHealTriggers",
    "BillingMeter",
    "BillingMeterCollection",
    "Capability",
    "Certificate",
    "CertificateCollection",
    "CertificatePatchResource",
    "CloningInfo",
    "ConnStringInfo",
    "CorsSettings",
    "CsmMoveResourceEnvelope",
    "CsmOperationCollection",
    "CsmOperationDescription",
    "CsmOperationDescriptionProperties",
    "CsmOperationDisplay",
    "CsmUsageQuota",
    "CsmUsageQuotaCollection",
    "DataSource",
    "DataTableResponseColumn",
    "DataTableResponseObject",
    "DefaultErrorResponse",
    "DefaultErrorResponseError",
    "DefaultErrorResponseErrorDetailsItem",
    "DeletedSite",
    "DeletedWebAppCollection",
    "DeploymentLocations",
    "DetectorAbnormalTimePeriod",
    "DetectorDefinition",
    "DetectorInfo",
    "DetectorResponse",
    "DetectorResponseCollection",
    "DiagnosticAnalysis",
    "DiagnosticAnalysisCollection",
    "DiagnosticCategory",
    "DiagnosticCategoryCollection",
    "DiagnosticData",
    "DiagnosticDetectorCollection",
    "DiagnosticDetectorResponse",
    "DiagnosticMetricSample",
    "DiagnosticMetricSet",
    "Dimension",
    "ErrorEntity",
    "Experiments",
    "GeoRegion",
    "GeoRegionCollection",
    "GlobalCsmSkuDescription",
    "HandlerMapping",
    "HostNameSslState",
    "HostingEnvironmentDeploymentInfo",
    "HostingEnvironmentProfile",
    "HybridConnection",
    "HybridConnectionKey",
    "Identifier",
    "IdentifierCollection",
    "IpSecurityRestriction",
    "LocalizableString",
    "ManagedServiceIdentity",
    "MetricAvailability",
    "MetricSpecification",
    "NameIdentifier",
    "NameValuePair",
    "NetworkAccessControlEntry",
    "Operation",
    "PremierAddOnOffer",
    "PremierAddOnOfferCollection",
    "ProxyOnlyResource",
    "PushSettings",
    "RampUpRule",
    "Recommendation",
    "RecommendationCollection",
    "RecommendationRule",
    "Rendering",
    "RequestsBasedTrigger",
    "Resource",
    "ResourceHealthMetadata",
    "ResourceHealthMetadataCollection",
    "ResourceMetric",
    "ResourceMetricAvailability",
    "ResourceMetricCollection",
    "ResourceMetricDefinition",
    "ResourceMetricDefinitionCollection",
    "ResourceMetricName",
    "ResourceMetricProperty",
    "ResourceMetricValue",
    "ResourceNameAvailability",
    "ResourceNameAvailabilityRequest",
    "ResponseMetaData",
    "ServiceSpecification",
    "Site",
    "SiteConfig",
    "SiteLimits",
    "SiteMachineKey",
    "SkuCapacity",
    "SkuDescription",
    "SkuInfos",
    "SlotSwapStatus",
    "SlowRequestsBasedTrigger",
    "SnapshotRecoveryRequest",
    "SnapshotRecoveryTarget",
    "Solution",
    "SourceControl",
    "SourceControlCollection",
    "StackMajorVersion",
    "StackMinorVersion",
    "StampCapacity",
    "StatusCodesBasedTrigger",
    "User",
    "ValidateRequest",
    "ValidateResponse",
    "ValidateResponseError",
    "VirtualApplication",
    "VirtualDirectory",
    "VirtualIPMapping",
    "VirtualNetworkProfile",
    "VnetGateway",
    "VnetInfo",
    "VnetParameters",
    "VnetRoute",
    "VnetValidationFailureDetails",
    "VnetValidationTestFailure",
    "WebAppCollection",
    "WorkerPool",
    "AccessControlEntryAction",
    "AppServicePlanRestrictions",
    "AutoHealActionType",
    "Channels",
    "CheckNameResourceTypes",
    "ComputeModeOptions",
    "ConnectionStringType",
    "Enum0",
    "Enum1",
    "HostType",
    "HostingEnvironmentStatus",
    "InAvailabilityReasonType",
    "InternalLoadBalancingMode",
    "IssueType",
    "KeyVaultSecretStatus",
    "ManagedPipelineMode",
    "ManagedServiceIdentityType",
    "NotificationLevel",
    "OperationStatus",
    "ProvisioningState",
    "RenderingType",
    "ResourceScopeType",
    "RouteType",
    "ScmType",
    "SiteAvailabilityState",
    "SiteLoadBalancing",
    "SkuName",
    "SolutionType",
    "SslState",
    "StatusOptions",
    "SupportedTlsVersions",
    "UsageState",
    "ValidateResourceTypes",
    "WorkerSizeOptions",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
