# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential
from azure.mgmt.dynatrace import DynatraceObservabilityMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-dynatrace
# USAGE
    python monitors_create_or_update_maximum_set_gen.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = DynatraceObservabilityMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="00000000-0000-0000-0000-000000000000",
    )

    response = client.monitors.begin_create_or_update(
        resource_group_name="myResourceGroup",
        monitor_name="myMonitor",
        resource={
            "identity": {"type": "SystemAssigned"},
            "location": "West US 2",
            "properties": {
                "dynatraceEnvironmentProperties": {
                    "accountInfo": {},
                    "environmentInfo": {},
                    "singleSignOnProperties": {},
                },
                "liftrResourceCategory": "Unknown",
                "marketplaceSubscriptionStatus": "Active",
                "monitoringStatus": "Enabled",
                "planData": {
                    "billingCycle": "Monthly",
                    "effectiveDate": "2019-08-30T15:14:33+02:00",
                    "planDetails": "dynatraceapitestplan",
                    "usageType": "Committed",
                },
                "provisioningState": "Accepted",
                "userInfo": {
                    "country": "westus2",
                    "emailAddress": "alice@microsoft.com",
                    "firstName": "Alice",
                    "lastName": "Bobab",
                    "phoneNumber": "123456",
                },
            },
            "tags": {"Environment": "Dev"},
        },
    ).result()
    print(response)


# x-ms-original-file: specification/dynatrace/resource-manager/Dynatrace.Observability/stable/2023-04-27/examples/Monitors_CreateOrUpdate_MaximumSet_Gen.json
if __name__ == "__main__":
    main()
