# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._operations import Operations
from ._workspaces_operations import WorkspacesOperations
from ._private_endpoint_connections_operations import PrivateEndpointConnectionsOperations
from ._private_link_resources_operations import PrivateLinkResourcesOperations
from ._scaling_plans_operations import ScalingPlansOperations
from ._scaling_plan_pooled_schedules_operations import ScalingPlanPooledSchedulesOperations
from ._scaling_plan_personal_schedules_operations import ScalingPlanPersonalSchedulesOperations
from ._application_groups_operations import ApplicationGroupsOperations
from ._start_menu_items_operations import StartMenuItemsOperations
from ._applications_operations import ApplicationsOperations
from ._desktops_operations import DesktopsOperations
from ._host_pools_operations import HostPoolsOperations
from ._user_sessions_operations import UserSessionsOperations
from ._session_hosts_operations import SessionHostsOperations
from ._msix_packages_operations import MSIXPackagesOperations
from ._app_attach_package_info_operations import AppAttachPackageInfoOperations
from ._msix_images_operations import MsixImagesOperations
from ._app_attach_package_operations import AppAttachPackageOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "WorkspacesOperations",
    "PrivateEndpointConnectionsOperations",
    "PrivateLinkResourcesOperations",
    "ScalingPlansOperations",
    "ScalingPlanPooledSchedulesOperations",
    "ScalingPlanPersonalSchedulesOperations",
    "ApplicationGroupsOperations",
    "StartMenuItemsOperations",
    "ApplicationsOperations",
    "DesktopsOperations",
    "HostPoolsOperations",
    "UserSessionsOperations",
    "SessionHostsOperations",
    "MSIXPackagesOperations",
    "AppAttachPackageInfoOperations",
    "MsixImagesOperations",
    "AppAttachPackageOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
