# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential
from azure.mgmt.costmanagement import CostManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-costmanagement
# USAGE
    python private_view_create_or_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = CostManagementClient(
        credential=DefaultAzureCredential(),
    )

    response = client.views.create_or_update(
        view_name="swaggerExample",
        parameters={
            "eTag": '"1d4ff9fe66f1d10"',
            "properties": {
                "accumulated": "true",
                "chart": "Table",
                "displayName": "swagger Example",
                "kpis": [
                    {"enabled": True, "id": None, "type": "Forecast"},
                    {
                        "enabled": True,
                        "id": "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/MYDEVTESTRG/providers/Microsoft.Consumption/budgets/swaggerDemo",
                        "type": "Budget",
                    },
                ],
                "metric": "ActualCost",
                "pivots": [
                    {"name": "ServiceName", "type": "Dimension"},
                    {"name": "MeterCategory", "type": "Dimension"},
                    {"name": "swaggerTagKey", "type": "TagKey"},
                ],
                "query": {
                    "dataSet": {
                        "aggregation": {"totalCost": {"function": "Sum", "name": "PreTaxCost"}},
                        "granularity": "Daily",
                        "grouping": [],
                        "sorting": [{"direction": "Ascending", "name": "UsageDate"}],
                    },
                    "timeframe": "MonthToDate",
                    "type": "Usage",
                },
            },
        },
    )
    print(response)


# x-ms-original-file: specification/cost-management/resource-manager/Microsoft.CostManagement/stable/2022-10-01/examples/PrivateViewCreateOrUpdate.json
if __name__ == "__main__":
    main()
