# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.datafactory.aio import DataFactoryManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDataFactoryManagementPrivateEndpointConnectionOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(DataFactoryManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_private_endpoint_connection_create_or_update(self, resource_group):
        response = await self.client.private_endpoint_connection.create_or_update(
            resource_group_name=resource_group.name,
            factory_name="str",
            private_endpoint_connection_name="str",
            private_endpoint_wrapper={
                "etag": "str",
                "id": "str",
                "name": "str",
                "properties": {
                    "privateEndpoint": {"id": "str"},
                    "privateLinkServiceConnectionState": {
                        "actionsRequired": "str",
                        "description": "str",
                        "status": "str",
                    },
                },
                "type": "str",
            },
            api_version="2018-06-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_private_endpoint_connection_get(self, resource_group):
        response = await self.client.private_endpoint_connection.get(
            resource_group_name=resource_group.name,
            factory_name="str",
            private_endpoint_connection_name="str",
            api_version="2018-06-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_private_endpoint_connection_delete(self, resource_group):
        response = await self.client.private_endpoint_connection.delete(
            resource_group_name=resource_group.name,
            factory_name="str",
            private_endpoint_connection_name="str",
            api_version="2018-06-01",
        )

        # please add some check logic here by yourself
        # ...
