# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.web import WebSiteManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-web
# USAGE
    python backup_web_app.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = WebSiteManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="34adfa4f-cedf-4dc0-ba29-b6d1a69ab345",
    )

    response = client.web_apps.backup(
        resource_group_name="testrg123",
        name="sitef6141",
        request={
            "properties": {
                "backupName": "abcdwe",
                "backupSchedule": {
                    "frequencyInterval": 7,
                    "frequencyUnit": "Day",
                    "keepAtLeastOneBackup": True,
                    "retentionPeriodInDays": 30,
                    "startTime": "2022-09-02T17:33:11.641Z",
                },
                "databases": [
                    {
                        "connectionString": "DSN=data-source-name[;SERVER=value] [;PWD=value] [;UID=value] [;<Attribute>=<value>]",
                        "connectionStringName": "backend",
                        "databaseType": "SqlAzure",
                        "name": "backenddb",
                    },
                    {
                        "connectionString": "DSN=data-source-name[;SERVER=value] [;PWD=value] [;UID=value] [;<Attribute>=<value>]",
                        "connectionStringName": "stats",
                        "databaseType": "SqlAzure",
                        "name": "statsdb",
                    },
                ],
                "enabled": True,
                "storageAccountUrl": "DefaultEndpointsProtocol=https;AccountName=storagesample;AccountKey=<account-key>",
            }
        },
    )
    print(response)


# x-ms-original-file: specification/web/resource-manager/Microsoft.Web/stable/2024-04-01/examples/BackupWebApp.json
if __name__ == "__main__":
    main()
