# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.compute import ComputeManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestComputeManagementProximityPlacementGroupsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ComputeManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_proximity_placement_groups_create_or_update(self, resource_group):
        response = self.client.proximity_placement_groups.create_or_update(
            resource_group_name=resource_group.name,
            proximity_placement_group_name="str",
            parameters={
                "location": "str",
                "availabilitySets": [
                    {
                        "colocationStatus": {
                            "code": "str",
                            "displayStatus": "str",
                            "level": "str",
                            "message": "str",
                            "time": "2020-02-20 00:00:00",
                        },
                        "id": "str",
                    }
                ],
                "colocationStatus": {
                    "code": "str",
                    "displayStatus": "str",
                    "level": "str",
                    "message": "str",
                    "time": "2020-02-20 00:00:00",
                },
                "id": "str",
                "intent": {"vmSizes": ["str"]},
                "name": "str",
                "proximityPlacementGroupType": "str",
                "tags": {"str": "str"},
                "type": "str",
                "virtualMachineScaleSets": [
                    {
                        "colocationStatus": {
                            "code": "str",
                            "displayStatus": "str",
                            "level": "str",
                            "message": "str",
                            "time": "2020-02-20 00:00:00",
                        },
                        "id": "str",
                    }
                ],
                "virtualMachines": [
                    {
                        "colocationStatus": {
                            "code": "str",
                            "displayStatus": "str",
                            "level": "str",
                            "message": "str",
                            "time": "2020-02-20 00:00:00",
                        },
                        "id": "str",
                    }
                ],
                "zones": ["str"],
            },
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_proximity_placement_groups_update(self, resource_group):
        response = self.client.proximity_placement_groups.update(
            resource_group_name=resource_group.name,
            proximity_placement_group_name="str",
            parameters={"tags": {"str": "str"}},
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_proximity_placement_groups_delete(self, resource_group):
        response = self.client.proximity_placement_groups.delete(
            resource_group_name=resource_group.name,
            proximity_placement_group_name="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_proximity_placement_groups_get(self, resource_group):
        response = self.client.proximity_placement_groups.get(
            resource_group_name=resource_group.name,
            proximity_placement_group_name="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_proximity_placement_groups_list_by_subscription(self, resource_group):
        response = self.client.proximity_placement_groups.list_by_subscription(
            api_version="2024-11-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_proximity_placement_groups_list_by_resource_group(self, resource_group):
        response = self.client.proximity_placement_groups.list_by_resource_group(
            resource_group_name=resource_group.name,
            api_version="2024-11-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...
