# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.automation.aio import AutomationClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAutomationWebhookOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AutomationClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_generate_uri(self, resource_group):
        response = await self.client.webhook.generate_uri(
            resource_group_name=resource_group.name,
            automation_account_name="str",
            api_version="2015-10-31",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_delete(self, resource_group):
        response = await self.client.webhook.delete(
            resource_group_name=resource_group.name,
            automation_account_name="str",
            webhook_name="str",
            api_version="2015-10-31",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_get(self, resource_group):
        response = await self.client.webhook.get(
            resource_group_name=resource_group.name,
            automation_account_name="str",
            webhook_name="str",
            api_version="2015-10-31",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_create_or_update(self, resource_group):
        response = await self.client.webhook.create_or_update(
            resource_group_name=resource_group.name,
            automation_account_name="str",
            webhook_name="str",
            parameters={
                "name": "str",
                "expiryTime": "2020-02-20 00:00:00",
                "isEnabled": bool,
                "parameters": {"str": "str"},
                "runOn": "str",
                "runbook": {"name": "str"},
                "uri": "str",
            },
            api_version="2015-10-31",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_update(self, resource_group):
        response = await self.client.webhook.update(
            resource_group_name=resource_group.name,
            automation_account_name="str",
            webhook_name="str",
            parameters={
                "description": "str",
                "isEnabled": bool,
                "name": "str",
                "parameters": {"str": "str"},
                "runOn": "str",
            },
            api_version="2015-10-31",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_list_by_automation_account(self, resource_group):
        response = self.client.webhook.list_by_automation_account(
            resource_group_name=resource_group.name,
            automation_account_name="str",
            api_version="2015-10-31",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...
