# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.appcontainers import ContainerAppsAPIClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestContainerAppsAPIConnectedEnvironmentsCertificatesOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ContainerAppsAPIClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_connected_environments_certificates_list(self, resource_group):
        response = self.client.connected_environments_certificates.list(
            resource_group_name=resource_group.name,
            connected_environment_name="str",
            api_version="2025-01-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_connected_environments_certificates_get(self, resource_group):
        response = self.client.connected_environments_certificates.get(
            resource_group_name=resource_group.name,
            connected_environment_name="str",
            certificate_name="str",
            api_version="2025-01-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_connected_environments_certificates_create_or_update(self, resource_group):
        response = self.client.connected_environments_certificates.create_or_update(
            resource_group_name=resource_group.name,
            connected_environment_name="str",
            certificate_name="str",
            api_version="2025-01-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_connected_environments_certificates_delete(self, resource_group):
        response = self.client.connected_environments_certificates.delete(
            resource_group_name=resource_group.name,
            connected_environment_name="str",
            certificate_name="str",
            api_version="2025-01-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_connected_environments_certificates_update(self, resource_group):
        response = self.client.connected_environments_certificates.update(
            resource_group_name=resource_group.name,
            connected_environment_name="str",
            certificate_name="str",
            certificate_envelope={"tags": {"str": "str"}},
            api_version="2025-01-01",
        )

        # please add some check logic here by yourself
        # ...
