# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._batch_account_operations import BatchAccountOperations
from ._application_package_operations import ApplicationPackageOperations
from ._application_operations import ApplicationOperations
from ._location_operations import LocationOperations
from ._operations import Operations
from ._certificate_operations import CertificateOperations
from ._private_link_resource_operations import PrivateLinkResourceOperations
from ._private_endpoint_connection_operations import PrivateEndpointConnectionOperations
from ._pool_operations import PoolOperations
from ._network_security_perimeter_operations import NetworkSecurityPerimeterOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "BatchAccountOperations",
    "ApplicationPackageOperations",
    "ApplicationOperations",
    "LocationOperations",
    "Operations",
    "CertificateOperations",
    "PrivateLinkResourceOperations",
    "PrivateEndpointConnectionOperations",
    "PoolOperations",
    "NetworkSecurityPerimeterOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
