# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import AvailableOperation
from ._models_py3 import AvailableOperationDisplay
from ._models_py3 import AvailableOperationDisplayPropertyServiceSpecificationMetricsItem
from ._models_py3 import AvailableOperationDisplayPropertyServiceSpecificationMetricsList
from ._models_py3 import AvailableOperationsListResponse
from ._models_py3 import CSRPError
from ._models_py3 import CSRPErrorBody
from ._models_py3 import CustomizationHostName
from ._models_py3 import CustomizationIPAddress
from ._models_py3 import CustomizationIPSettings
from ._models_py3 import CustomizationIdentity
from ._models_py3 import CustomizationIdentityUserData
from ._models_py3 import CustomizationNicSetting
from ._models_py3 import CustomizationPoliciesListResponse
from ._models_py3 import CustomizationPolicy
from ._models_py3 import CustomizationSpecification
from ._models_py3 import DedicatedCloudNode
from ._models_py3 import DedicatedCloudNodeListResponse
from ._models_py3 import DedicatedCloudService
from ._models_py3 import DedicatedCloudServiceListResponse
from ._models_py3 import GuestOSCustomization
from ._models_py3 import GuestOSNICCustomization
from ._models_py3 import OperationError
from ._models_py3 import OperationResource
from ._models_py3 import PatchPayload
from ._models_py3 import PrivateCloud
from ._models_py3 import PrivateCloudList
from ._models_py3 import ResourcePool
from ._models_py3 import ResourcePoolsListResponse
from ._models_py3 import Sku
from ._models_py3 import SkuAvailability
from ._models_py3 import SkuAvailabilityListResponse
from ._models_py3 import Usage
from ._models_py3 import UsageListResponse
from ._models_py3 import UsageName
from ._models_py3 import VirtualDisk
from ._models_py3 import VirtualDiskController
from ._models_py3 import VirtualMachine
from ._models_py3 import VirtualMachineListResponse
from ._models_py3 import VirtualMachineStopMode
from ._models_py3 import VirtualMachineTemplate
from ._models_py3 import VirtualMachineTemplateListResponse
from ._models_py3 import VirtualNetwork
from ._models_py3 import VirtualNetworkListResponse
from ._models_py3 import VirtualNic

from ._vmware_cloud_simple_enums import AggregationType
from ._vmware_cloud_simple_enums import CustomizationHostNameType
from ._vmware_cloud_simple_enums import CustomizationIPAddressType
from ._vmware_cloud_simple_enums import CustomizationIdentityType
from ._vmware_cloud_simple_enums import CustomizationPolicyPropertiesType
from ._vmware_cloud_simple_enums import DiskIndependenceMode
from ._vmware_cloud_simple_enums import GuestOSNICCustomizationAllocation
from ._vmware_cloud_simple_enums import GuestOSType
from ._vmware_cloud_simple_enums import NICType
from ._vmware_cloud_simple_enums import NodeStatus
from ._vmware_cloud_simple_enums import OnboardingStatus
from ._vmware_cloud_simple_enums import OperationOrigin
from ._vmware_cloud_simple_enums import StopMode
from ._vmware_cloud_simple_enums import UsageCount
from ._vmware_cloud_simple_enums import VirtualMachineStatus
from ._patch import __all__ as _patch_all
from ._patch import *  # type: ignore # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AvailableOperation",
    "AvailableOperationDisplay",
    "AvailableOperationDisplayPropertyServiceSpecificationMetricsItem",
    "AvailableOperationDisplayPropertyServiceSpecificationMetricsList",
    "AvailableOperationsListResponse",
    "CSRPError",
    "CSRPErrorBody",
    "CustomizationHostName",
    "CustomizationIPAddress",
    "CustomizationIPSettings",
    "CustomizationIdentity",
    "CustomizationIdentityUserData",
    "CustomizationNicSetting",
    "CustomizationPoliciesListResponse",
    "CustomizationPolicy",
    "CustomizationSpecification",
    "DedicatedCloudNode",
    "DedicatedCloudNodeListResponse",
    "DedicatedCloudService",
    "DedicatedCloudServiceListResponse",
    "GuestOSCustomization",
    "GuestOSNICCustomization",
    "OperationError",
    "OperationResource",
    "PatchPayload",
    "PrivateCloud",
    "PrivateCloudList",
    "ResourcePool",
    "ResourcePoolsListResponse",
    "Sku",
    "SkuAvailability",
    "SkuAvailabilityListResponse",
    "Usage",
    "UsageListResponse",
    "UsageName",
    "VirtualDisk",
    "VirtualDiskController",
    "VirtualMachine",
    "VirtualMachineListResponse",
    "VirtualMachineStopMode",
    "VirtualMachineTemplate",
    "VirtualMachineTemplateListResponse",
    "VirtualNetwork",
    "VirtualNetworkListResponse",
    "VirtualNic",
    "AggregationType",
    "CustomizationHostNameType",
    "CustomizationIPAddressType",
    "CustomizationIdentityType",
    "CustomizationPolicyPropertiesType",
    "DiskIndependenceMode",
    "GuestOSNICCustomizationAllocation",
    "GuestOSType",
    "NICType",
    "NodeStatus",
    "OnboardingStatus",
    "OperationOrigin",
    "StopMode",
    "UsageCount",
    "VirtualMachineStatus",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
