# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.appplatform import AppPlatformManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-appplatform
# USAGE
    python customized_accelerators_create_or_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = AppPlatformManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="00000000-0000-0000-0000-000000000000",
    )

    response = client.customized_accelerators.begin_create_or_update(
        resource_group_name="myResourceGroup",
        service_name="myservice",
        application_accelerator_name="default",
        customized_accelerator_name="acc-name",
        customized_accelerator_resource={
            "properties": {
                "acceleratorTags": ["tag-a", "tag-b"],
                "description": "acc-desc",
                "displayName": "acc-name",
                "gitRepository": {
                    "authSetting": {
                        "authType": "SSH",
                        "hostKey": "git-auth-hostkey",
                        "hostKeyAlgorithm": "git-auth-algorithm",
                        "privateKey": "git-auth-privatekey",
                    },
                    "branch": "git-branch",
                    "commit": "12345",
                    "gitTag": "git-tag",
                    "intervalInSeconds": 70,
                    "url": "git-url",
                },
                "iconUrl": "acc-icon",
            },
            "sku": {"capacity": 2, "name": "E0", "tier": "Enterprise"},
        },
    ).result()
    print(response)


# x-ms-original-file: specification/appplatform/resource-manager/Microsoft.AppPlatform/stable/2023-12-01/examples/CustomizedAccelerators_CreateOrUpdate.json
if __name__ == "__main__":
    main()
