# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._bots_operations import BotsOperations
from ._channels_operations import ChannelsOperations
from ._direct_line_operations import DirectLineOperations
from ._email_operations import EmailOperations
from ._operations import Operations
from ._bot_connection_operations import BotConnectionOperations
from ._qn_amaker_endpoint_keys_operations import QnAMakerEndpointKeysOperations
from ._host_settings_operations import HostSettingsOperations
from ._operation_results_operations import OperationResultsOperations
from ._private_endpoint_connections_operations import PrivateEndpointConnectionsOperations
from ._private_link_resources_operations import PrivateLinkResourcesOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "BotsOperations",
    "ChannelsOperations",
    "DirectLineOperations",
    "EmailOperations",
    "Operations",
    "BotConnectionOperations",
    "QnAMakerEndpointKeysOperations",
    "HostSettingsOperations",
    "OperationResultsOperations",
    "PrivateEndpointConnectionsOperations",
    "PrivateLinkResourcesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
