# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.apimanagement import ApiManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestApiManagement(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ApiManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_perform_connectivity_check_async(self, resource_group):
        response = self.client.begin_perform_connectivity_check_async(
            resource_group_name=resource_group.name,
            service_name="str",
            connectivity_check_request_params={
                "destination": {"address": "str", "port": 0},
                "source": {"region": "str", "instance": 0},
                "preferredIPVersion": "str",
                "protocol": "str",
                "protocolConfiguration": {
                    "HTTPConfiguration": {
                        "headers": [{"name": "str", "value": "str"}],
                        "method": "str",
                        "validStatusCodes": [0],
                    }
                },
            },
            api_version="2024-05-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...
