# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._operations import Operations
from ._dedicated_cloud_nodes_operations import DedicatedCloudNodesOperations
from ._dedicated_cloud_services_operations import DedicatedCloudServicesOperations
from ._skus_availability_operations import SkusAvailabilityOperations
from ._private_clouds_operations import PrivateCloudsOperations
from ._customization_policies_operations import CustomizationPoliciesOperations
from ._resource_pools_operations import ResourcePoolsOperations
from ._virtual_machine_templates_operations import VirtualMachineTemplatesOperations
from ._virtual_networks_operations import VirtualNetworksOperations
from ._usages_operations import UsagesOperations
from ._virtual_machines_operations import VirtualMachinesOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # type: ignore # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "DedicatedCloudNodesOperations",
    "DedicatedCloudServicesOperations",
    "SkusAvailabilityOperations",
    "PrivateCloudsOperations",
    "CustomizationPoliciesOperations",
    "ResourcePoolsOperations",
    "VirtualMachineTemplatesOperations",
    "VirtualNetworksOperations",
    "UsagesOperations",
    "VirtualMachinesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
